@@ -46,6 +46,13 @@ func renderKuryr(conf *operv1.NetworkSpec, bootstrapResult *bootstrap.BootstrapR
 	data.Data["OpenStackCloud"] = b.OpenStackCloud
 	// FIXME(dulek): Move that logic to the template once it's known how to dereference pointers there.
 	data.Data["OpenStackInsecureAPI"] = b.OpenStackCloud.Verify != nil && !*b.OpenStackCloud.Verify
+	data.Data["DebugEnabled"] = conf.LogLevel != operv1.Normal
+
+	// kuryr port pools options
+	data.Data["EnablePortPoolsPrepopulation"] = c.EnablePortPoolsPrepopulation
+	data.Data["PoolMaxPorts"] = c.PoolMaxPorts
+	data.Data["PoolMinPorts"] = c.PoolMinPorts
+	data.Data["PoolBatchPorts"] = c.PoolBatchPorts
 
 	// kuryr-daemon DaemonSet data
 	data.Data["DaemonEnableProbes"] = true
@@ -177,4 +184,13 @@ func fillKuryrDefaults(conf *operv1.NetworkSpec) {
 		octaviaServiceNet := iputil.ExpandNet(*svcNet)
 		kc.OpenStackServiceNetwork = octaviaServiceNet.String()
 	}
+
+	if kc.PoolMinPorts == 0 {
+		kc.PoolMinPorts = 1
+	}
+
+	if kc.PoolBatchPorts == nil {
+		var batchPorts uint = 3
+		kc.PoolBatchPorts = &batchPorts
+	}
 }
